SubSuperRecord

Struct SubSuperRecord 

pub struct SubSuperRecord(pub Vec<Record>);
A set of Record mapping to complex entity instance, e.g. (A(1) B(2.0) C("3"))

FromStr

use std::str::FromStr;
use step_p21::ast::*;

let record = SubSuperRecord::from_str("(A(1, 2) B(3, 4))").unwrap();
assert_eq!(
    record,
    SubSuperRecord(vec![
        Record {
            name: "A".to_string(),
            parameter: vec![Parameter::Integer(1), Parameter::Integer(2)]
                .into(),
        },
        Record {
            name: "B".to_string(),
            parameter: vec![Parameter::Integer(3), Parameter::Integer(4)]
                .into(),
        }
    ])
)

Deserialize as a map

Similar to Record, SubSuperRecord can be deserialized as a “map”:

use serde::Deserialize;
use std::{collections::HashMap, str::FromStr};
use step_p21::ast::*;

let p = SubSuperRecord::from_str("(A(1, 2) B(3, 4))").unwrap();

// Map can be deserialize as a hashmap
assert_eq!(
    HashMap::<String, Vec<i32>>::deserialize(&p).unwrap(),
    maplit::hashmap! {
        "A".to_string() => vec![1, 2],
        "B".to_string() => vec![3, 4],
    }
);

// Map in serde can be interpreted as Rust field
#[derive(Debug, Clone, PartialEq, Deserialize)]
struct X {
    #[serde(rename = "A")]
    a: Vec<i32>,
    #[serde(rename = "B")]
    b: Vec<i32>,
}
assert_eq!(
    X::deserialize(&p).unwrap(),
    X {
        a: vec![1, 2],
        b: vec![3, 4]
    }
);

External mapping to complex entity instance

As discussed in Record for internal mapping,

ENTITY person;
  name: STRING;
END_ENTITY;

ENTITY employee SUBTYPE OF (person);
  pay: INTEGER;
END_ENTITY;

ENTITY student SUBTYPE OF (person);
  school_name: STRING;
END_ENTITY;

The instance of person may have three partial complex entity. External mapping has different looks in exchange structure:

#3 = (PERSON('Hitori Goto') EMPLOYEE(10));
#4 = (PERSON('Ikuno Kita') STUDENT('Shuka));
#5 = (PERSON('Nizika Iziti') EMPLOYEE(15) STUDENT('Simokitazawa'))

Each components enclosed by () corresponds to a partial entity instance, which consists of fields declared in its entity definition.

#5 = (PERSON('Nizika Iziti') EMPLOYEE(15) STUDENT('Simokitazawa'))
             ▲                        ▲           ▲
             │                        │           └─ school_name in student
             │                        └─ pay in employee
             └─ name in person

The detail of external mapping is defined in 12.2.5.3 “External mapping” of ISO-10303-21.

SubSuperRecord is not self-describing because EXPRESS does not defines memory layout of complex entities.

Tuple Fields

0: Vec<Record>