Header section of exchange structure
HEADER section of exchange structure shall contains
one instance of each of following entities in this order:
file_descriptionfile_namefile_schema
and following entities may appear after file_schema:
schema_populationfile_populationsection_languagesection_context
These entities are defined in ISO-10303-21 “8.2 Header section declarations” using EXPRESS schemas. Although we can generate corresponding Rust struct using step_p11 compiler, we write these definitions manually to keep development process simple.
Structs
- File
Description - File description
- File
Name - File name
- File
Schema - File schema
- Header
- STEP-file HEADER section